﻿using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public class NormalIntValueCrossover : SingleSuccessorOperator, IIntValueCrossover, IStochasticOperator {
    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    public NormalIntValueCrossover() { }
    [StorableConstructor]
    protected NormalIntValueCrossover(bool deserializing) : base(deserializing) { }
    protected NormalIntValueCrossover(NormalIntValueCrossover original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new NormalIntValueCrossover(this, cloner);
    }

    public void Apply(IRandom random, IntValue value, IntValue other, IntValueRange range) {
      value.Value = ApplyStatic(random, value, other, range).Value;
    }

    public static IntValue ApplyStatic(IRandom random, IntValue better, IntValue worse, IntValueRange range) {
      NormalDistributedRandom N = new NormalDistributedRandom(random, better.Value, Math.Abs(better.Value - worse.Value) / 3);
      var offspring = new IntValue();
      do {
        offspring.Value = (int)N.NextDouble();
        offspring.Value = range.ApplyStepSize(offspring.Value);
      } while (!range.IsInRange(offspring.Value));
      return offspring;
    }
  }
}
